/*
 * Copyright (c) 1997  Motoyuki Kasahara
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <sys/types.h>
#include <syslog.h>

#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
#include <string.h>
#if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
#include <memory.h>
#endif /* not STDC_HEADERS and HAVE_MEMORY_H */
#else /* not STDC_HEADERS and not HAVE_STRING_H */
#include <strings.h>
#endif /* not STDC_HEADERS and not HAVE_STRING_H */

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifndef HAVE_STRCHR
#define strchr index
#define strrchr rindex
#endif /* HAVE_STRCHR */

#include "fakelog.h"
#include "getopt.h"
#include "hostname.h"
#include "permission.h"
#include "readconf.h"
#include "wildcard.h"

#include "ndtpd.h"

/*
 * Unexported functions.
 */
#ifdef __STDC__
static void output_version(void);
static void output_help(void);
static void output_try_help(void);
#else
static void output_version();
static void output_help();
static void output_try_help();
#endif

/*
 * Program name.
 */
#define PROGRAM_NAME	"ndtpcheck"

/*
 * Long style options.
 */
static struct option longopts[] = {
    {"configuration-file", required_argument, NULL, 'c'},
    {"debug",              no_argument,       NULL, 'd'},
    {"help",               no_argument,       NULL, 'h'},
    {"version",            no_argument,       NULL, 'v'},
    {NULL, 0, NULL, 0}
};


int
main(argc, argv)
    int argc;
    char *argv[];
{
    int ch;

    /*
     * Set program name and version.
     */
    invoked_name = argv[0];
    program_name = PROGRAM_NAME;
    program_version = PROGRAM_VERSION;

    /*
     * Initialize global and static variables.
     */
    server_mode = SERVER_MODE_UTILITY;
    initialize_permission(&permissions);
    initialize_permission(&identifications);
    initialize_book_registry();
    strcpy(configuration_filename, DEFAULT_CONFIG_FILENAME);
    set_fakelog(FAKELOG_TO_STDERR, LOG_ERR);

    /*
     * Parse command line options.
     */
    for (;;) {
	ch = getopt_long(argc, argv, "c:dhv", longopts, NULL);
	if (ch == EOF)
	    break;
	switch (ch) {
	case 'c':
	    /*
	     * Option `-c file'.  Specify a configuration filename.
	     */
	    if (MAXLEN_FILENAME < strlen(optarg)) {
		fprintf(stderr, "%s: too long configuration filename\n",
		    invoked_name);
		exit(1);
	    }
	    strcpy(configuration_filename, optarg);
	    break;

	case 'd':
	    /*
	     * Option `-d'.  Debug mode.
	     */
	    set_fakelog(FAKELOG_TO_STDERR, LOG_DEBUG);
	    break;

	case 'h':
	    /*
	     * Option `-h'.  Help.
	     */
	    output_help();
	    exit(0);

	case 'v':
	    /*
	     * Option `-v'.  Show the version number.
	     */
	    output_version();
	    exit(0);

	default:
	    output_try_help();
	    exit(1);
	}
    }

    /*
     * Check for the number of rest arguments.
     */
    if (0 < argc - optind) {
	fprintf(stderr, "%s: too many arguments\n", invoked_name);
	output_try_help();
	exit(1);
    }

    /*
     * Read the configuration file.
     */
    if (read_configuration(configuration_filename, configuration_table) < 0) {
	fprintf(stderr, "configuration failure\n");
	exit(1);
    }

    return 0;
}


/*
 * Output the version number to stderr.
 */
static void
output_version()
{
    fprintf(stderr, "%s version %s\n", program_name, program_version);
    fflush(stderr);
}


/*
 * Output the usage to stderr.
 */
static void
output_help()
{
    fprintf(stderr, "Usage: %s [option...]\n", program_name);
    fprintf(stderr, "Options:\n");
    fprintf(stderr, "  -c FILE, --configuration-file FILE\n");
    fprintf(stderr, "                             specify a configuration file\n");
    fprintf(stderr, "                             (default: %s)\n",
	DEFAULT_CONFIG_FILENAME);
    fprintf(stderr, "  -d, --debug                degug mode\n");
    fprintf(stderr, "  -h, --help                 display this help, then exit\n");
    fprintf(stderr, "  -v, --version              display version number, then exit\n");
    fprintf(stderr, "\nDefault value in a configuration file:\n");
    fprintf(stderr, "  work-path                  (default: %s)\n", 
	DEFAULT_WORK_PATH);
    fflush(stderr);
}


/*
 * Output ``try ...'' message to standard error.
 */
static void
output_try_help()
{
    fprintf(stderr, "try `%s --help' for more information\n", invoked_name);
    fflush(stderr);
}


